@@ -9,24 +9,18 @@ set -o errexit
 set -o nounset
 set -o pipefail
 
+PROJECT_ROOT=$(cd $(dirname ${BASH_SOURCE})/..; pwd)
+PATH="${PROJECT_ROOT}/dist:${PATH}"
+GOPATH=$(go env GOPATH)
+
 # output tool versions
+go version
 protoc --version
 swagger version
 jq --version
 
-PROJECT_ROOT=$(cd $(dirname ${BASH_SOURCE})/..; pwd)
-CODEGEN_PKG=${CODEGEN_PKG:-$(cd ${PROJECT_ROOT}; ls -d -1 ./vendor/k8s.io/code-generator 2>/dev/null || echo ../code-generator)}
-PATH="${PROJECT_ROOT}/dist:${PATH}"
-MOD_ROOT=${GOPATH}/pkg/mod
-
-. ${PROJECT_ROOT}/hack/versions.sh
-
 export GO111MODULE=off
 
-# protobuf tooling required to build .proto files from go annotations from k8s-like api types
-go build -o dist/go-to-protobuf ./vendor/k8s.io/code-generator/cmd/go-to-protobuf
-go build -o dist/protoc-gen-gogo ./vendor/k8s.io/code-generator/cmd/go-to-protobuf/protoc-gen-gogo
-
 # Generate pkg/apis/<group>/<apiversion>/(generated.proto,generated.pb.go)
 # NOTE: any dependencies of our types to the k8s.io apimachinery types should be added to the
 # --apimachinery-packages= option so that go-to-protobuf can locate the types, but prefixed with a
@@ -41,44 +35,49 @@ APIMACHINERY_PKGS=(
     +k8s.io/apimachinery/pkg/runtime
     k8s.io/apimachinery/pkg/apis/meta/v1
     k8s.io/api/core/v1
+    k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1
 )
 
 export GO111MODULE=on
 [ -e ./v2 ] || ln -s . v2
 
-${PROJECT_ROOT}/dist/go-to-protobuf \
+# protoc_include is the include directory containing the .proto files distributed with protoc binary
+if [ -d /dist/protoc-include ]; then
+    # containerized codegen build
+    protoc_include=/dist/protoc-include
+else
+    # local codegen build
+    protoc_include=${PROJECT_ROOT}/dist/protoc-include
+fi
+
+go-to-protobuf \
     --go-header-file=${PROJECT_ROOT}/hack/custom-boilerplate.go.txt \
     --packages=$(IFS=, ; echo "${PACKAGES[*]}") \
     --apimachinery-packages=$(IFS=, ; echo "${APIMACHINERY_PKGS[*]}") \
-    --proto-import=./vendor
+    --proto-import=./vendor \
+    --proto-import=${protoc_include}
 
 # Either protoc-gen-go, protoc-gen-gofast, or protoc-gen-gogofast can be used to build
 # server/*/<service>.pb.go from .proto files. golang/protobuf and gogo/protobuf can be used
 # interchangeably. The difference in the options are:
 # 1. protoc-gen-go - official golang/protobuf
-#go build -o dist/protoc-gen-go ./vendor/github.com/golang/protobuf/protoc-gen-go
 #GOPROTOBINARY=go
 # 2. protoc-gen-gofast - fork of golang golang/protobuf. Faster code generation
-#go build -o dist/protoc-gen-gofast ./vendor/github.com/gogo/protobuf/protoc-gen-gofast
 #GOPROTOBINARY=gofast
 # 3. protoc-gen-gogofast - faster code generation and gogo extensions and flexibility in controlling
 # the generated go code (e.g. customizing field names, nullable fields)
-go build -o dist/protoc-gen-gogofast ./vendor/github.com/gogo/protobuf/protoc-gen-gogofast
 GOPROTOBINARY=gogofast
 
-# protoc-gen-grpc-gateway is used to build <service>.pb.gw.go files from from .proto files
-go build -o dist/protoc-gen-grpc-gateway ./vendor/github.com/grpc-ecosystem/grpc-gateway/protoc-gen-grpc-gateway
-# protoc-gen-swagger is used to build swagger.json
-go build -o dist/protoc-gen-swagger ./vendor/github.com/grpc-ecosystem/grpc-gateway/protoc-gen-swagger
-
 # Generate server/<service>/(<service>.pb.go|<service>.pb.gw.go)
-PROTO_FILES=$(find $PROJECT_ROOT \( -name "*.proto" -and -path '*/server/*' -or -path '*/reposerver/*' -and -name "*.proto" \) | sort)
+MOD_ROOT=${GOPATH}/pkg/mod
+grpc_gateway_version=$(go list -m github.com/grpc-ecosystem/grpc-gateway | awk '{print $NF}' | head -1)
+GOOGLE_PROTO_API_PATH=${MOD_ROOT}/github.com/grpc-ecosystem/grpc-gateway@${grpc_gateway_version}/third_party/googleapis
+GOGO_PROTOBUF_PATH=${PROJECT_ROOT}/vendor/github.com/gogo/protobuf
+PROTO_FILES=$(find $PROJECT_ROOT \( -name "*.proto" -and -path '*/server/*' -or -path '*/reposerver/*' -and -name "*.proto" -or -path '*/cmpserver/*' -and -name "*.proto" \) | sort)
 for i in ${PROTO_FILES}; do
-    GOOGLE_PROTO_API_PATH=${MOD_ROOT}/github.com/grpc-ecosystem/grpc-gateway@${grpc_gateway_version}/third_party/googleapis
-    GOGO_PROTOBUF_PATH=${PROJECT_ROOT}/vendor/github.com/gogo/protobuf
     protoc \
         -I${PROJECT_ROOT} \
-        -I/usr/local/include \
+        -I${protoc_include} \
         -I./vendor \
         -I$GOPATH/src \
         -I${GOOGLE_PROTO_API_PATH} \
@@ -125,8 +124,9 @@ clean_swagger() {
 }
 
 echo "If additional types are added, the number of expected collisions may need to be increased"
-EXPECTED_COLLISION_COUNT=55
+EXPECTED_COLLISION_COUNT=90
 collect_swagger server ${EXPECTED_COLLISION_COUNT}
 clean_swagger server
 clean_swagger reposerver
 clean_swagger controller
+clean_swagger cmpserver
